import * as React from "react";
import {Tab, Tabs} from "@material-ui/core";
import {RxModuleComponentModel} from "../../stores/RxModuleComponentStore";
import {observer} from "mobx-react-lite";
import {ComponentPermissionTab} from "./ComponentPermissionTab";
import {Permission} from "../../stores/RoleStore";
import {moduleEnabled, moduleFeatureEnabled} from "@rx-suite/stores";

type ComponentPermissionsFormProps = {
    permissions: Array<Permission>
    apps: Array<RxModuleComponentModel>
}

type AppsPermissionGroup = {
    id: string
    module_name: string,
    components: Array<ComponentPermissionGroup>
}

export type ComponentPermissionGroup = {
    target: {
        target_id: string,
        target_display_name: string,
        target_category: string
    },
    actions: Array<Permission>
}

export const ComponentPermissionForm: React.FunctionComponent<ComponentPermissionsFormProps> = observer((props) => {
    const [value, setValue] = React.useState(0);
    const handleChange = (event: any, newValue: React.SetStateAction<number>) => {
        setValue(newValue);
    };

    const {permissions, apps} = props;

    // Group the permission by app and component
    const appsData: Array<AppsPermissionGroup> = apps.map(app => {
        const appComponents = permissions
            .filter((permission) => permission.componentPermission.component_id.toString() === app.getId().toString())
            .filter((permission) => {
                if (permission.componentPermission.feature_module_id) {
                    if (permission.componentPermission.feature_subfeature_id) {
                        return moduleFeatureEnabled(permission.componentPermission.feature_module_id, permission.componentPermission.feature_subfeature_id);
                    } else {
                        return moduleEnabled(permission.componentPermission.feature_module_id);
                    }
                }
                return true;
            })
            .reduce((acc: Array<any>, permission) => {
                let index = acc.map(component => component.target.target_id.toString()).indexOf(permission.componentPermission.target_id.toString());

                // component does not exists, create it
                if (index === -1) {
                    acc.push({
                        target: {
                            target_id: permission.componentPermission.target_id,
                            target_display_name: permission.componentPermission.target_display_name,
                            target_category: permission.componentPermission.target_category
                        },
                        actions: []
                    });

                    index = acc.length - 1;
                }

                // add a new action
                acc[index].actions.push({
                    componentPermission: permission.componentPermission,
                    rolePermission: permission.rolePermission
                });

                return acc;

            }, []);

        return {
            id: app.id as string,
            module_name: app.module_name as string,
            components: appComponents
        }
    });

    return (
        <div>
            <Tabs
                value={value}
                onChange={handleChange}
                indicatorColor="primary"
                textColor="primary"
                variant="scrollable"
            >
                {appsData.sort((a, b) => {
                    const aKey = (a || {}).module_name || '';
                    const bKey = (b || {}).module_name || '';
                    return aKey.localeCompare(bKey)
                }).map(app => {
                    return <Tab label={app.module_name} key={app.id as string}/>
                })}
            </Tabs>

            {appsData.map((app, i) => {
                return <ComponentPermissionTab
                    key={app.id as string}
                    components={app.components}
                    value={value}
                    index={i}
                />
            })}
        </div>
    );
});