import * as React from "react";
import {observer} from "mobx-react-lite";
import {t, Trans} from "@lingui/macro";
import Calendar from "@toast-ui/react-calendar";
import {ShiftStore} from "../../stores/ShiftStore";
import {ButtonField, SelectField} from "@rx-suite/ui";
import {makeStyles} from "@material-ui/core/styles";
import {CalendarConfigButton} from "./CalendarConfigButton";
import {IconButton, LinearProgress, Typography} from "@material-ui/core";
import {Replay} from "@material-ui/icons";
import {Alert} from "@material-ui/lab";
import moment from "moment";

const useStyles = makeStyles(theme => ({
    commandBar: {
        display: "flex",
        width: '100%',
        alignItems: 'center'
    },
    title: {
        marginRight: theme.spacing(1)
    },
    loadbar: {
        width: '100%'
    },
    viewPicker: {
        flexGrow: 1
    }
}));

export type ViewMode = 'day' | 'week' | 'month';

type CommandBarProps = {
    view: ViewMode,
    setView: (view: ViewMode) => void,

    loading: boolean,
    loaderr: string,

    bustCache: () => void,
    calendarRefContainer: React.RefObject<Calendar>,

    hasPendingChanges?: boolean
};

export const CalendarCommandBar: React.FunctionComponent<CommandBarProps> = observer(({
                                                                                          view,
                                                                                          setView,
                                                                                          loading,
                                                                                          loaderr,
                                                                                          calendarRefContainer,
                                                                                          bustCache,
                                                                                          hasPendingChanges
                                                                                      }) => {
    const classes = useStyles();
    const handleViewModeChange = React.useCallback((_name, value) => {
        setView(value);
        bustCache();
    }, [setView, bustCache]);
    const goToToday = React.useCallback(() => {
        const calendar = calendarRefContainer.current as Calendar | undefined;
        if (calendar) {
            calendar.getInstance().today();
            bustCache();
        }
    }, [calendarRefContainer, bustCache]);
    const goPrevious = React.useCallback(() => {
        const calendar = calendarRefContainer.current as Calendar | undefined;
        if (calendar) {
            calendar.getInstance().prev();
            bustCache();
        }
    }, [calendarRefContainer, bustCache]);
    const goNext = React.useCallback(() => {
        const calendar = calendarRefContainer.current as Calendar | undefined;
        if (calendar) {
            calendar.getInstance().next();
            bustCache();
        }
    }, [calendarRefContainer, bustCache]);
    const refresh = React.useCallback(() => {
        bustCache();
    }, [bustCache]);

    let dateRangeText = '';
    if (calendarRefContainer.current) {
        const calendarInstance = calendarRefContainer.current.getInstance();
        if (view === 'month') {
            const calDate = moment(calendarInstance.getDate().toDate());
            dateRangeText = calDate.format("MMMM YYYY");
        } else if (view === 'week') {
            const rangeStart = moment(calendarInstance.getDateRangeStart().toDate());
            const rangeEnd = moment(calendarInstance.getDateRangeEnd().toDate());
            const start = rangeStart.format("YYYY/MM/DD");
            const end = rangeEnd.format("YYYY/MM/DD");
            dateRangeText = `${start} - ${end}`;
        }
    }

    return (
        <>
            {loaderr && <Alert severity="error">
                <Typography variant={'h6'}>
                    <Trans>The following error occurred when refreshing the calendar: </Trans>
                </Typography>
                {loaderr}
            </Alert>}
            {hasPendingChanges && <Alert severity="warning"><Typography variant={'h6'}>
                <Trans>There are unsaved changes. This calendar will not update until the shift is saved.</Trans>
            </Typography></Alert>}
            <div className={classes.commandBar}>
                {dateRangeText && <div className={classes.title}>
                    <Typography variant={'h6'}>{dateRangeText}</Typography>
                </div>}
                <div className={classes.viewPicker}>
                    <SelectField label={t`View`}
                                 onChange={handleViewModeChange}
                                 value={view}
                                 options={[
                                     {label: t`Day`, value: 'day'},
                                     {label: t`Week`, value: 'week'},
                                     {label: t`Month`, value: 'month'},
                                 ]}
                    />
                </div>
                <ButtonField text={t`Today`} onClick={goToToday}/>
                <ButtonField text={t`Previous`} onClick={goPrevious}/>
                <ButtonField text={t`Next`} onClick={goNext}/>
                <IconButton onClick={refresh}>
                    <Replay/>
                </IconButton>
                <CalendarConfigButton/>
            </div>
            {loading && <div className={classes.loadbar}>
                <LinearProgress/>
            </div>}
        </>
    );
});