import * as React from 'react';
import {observer} from "mobx-react-lite";
import {ButtonField, FormProps, ModelForm, ModelInput, ModelSelect} from "@rx-suite/ui";
import {t} from '@lingui/macro';
import Typography from '@material-ui/core/Typography';
import {Box, Grid, Table, TableBody, TableCell, TableRow, Toolbar} from '@material-ui/core';
import {EmailSelect} from "../../components/emails/EmailSelect";
import {UpdateCell} from "../../components/apps/UpdateCell";
import {RxModuleComponentModel, RxModuleComponentStore} from "../../stores/RxModuleComponentStore";
import {useSnackbar} from "notistack";
import {errToString} from "@rx-suite/data";
import {AppStatusIcon} from "./AppStatusIcon";
import {StopButton} from "./StopButton";
import {RestartButton} from "./RestartButton";

export const AppForm: React.FunctionComponent<FormProps> = observer(({model, readOnly}) => {
        const {enqueueSnackbar} = useSnackbar();
        const updateIcon = React.useCallback(async () => {
            try {
                enqueueSnackbar(t`Updating icon...`, {variant: "info"});
                await RxModuleComponentStore.transportLayer.fetch(`/${model.getId()}/update_icon`, {method: "POST"});
                enqueueSnackbar(t`Icon updated`, {variant: "success"});
            } catch (e) {
                enqueueSnackbar(t`Icon update failed:` + errToString(e), {variant: "error"});
            }
        }, [model])
        return (
            <Grid container spacing={2}>
                <Grid item md={model.isNew ? 12 : 6} xs={12}>
                    <ModelForm model={model} readOnly={readOnly} suppressWrapper>
                        <ModelSelect name={"type"} options={[
                            {label: t`Rx-Module`, value: 'RxModuleComponent'},
                            {label: t`Rx-Universe API`, value: 'RxUniverseApiComponent'},
                            {label: t`MES API`, value: 'MesApiComponent'},
                            {label: t`Router API`, value: 'RouterApiComponent'}
                        ]}/>
                        <ModelSelect name={"protocol"} options={[
                            {label: t`HTTP`, value: 'http'},
                            {label: t`HTTPS`, value: 'https'}
                        ]}/>
                        <ModelInput name={"port"}/>
                        <ModelInput name={"ip_address"}/>
                        <EmailSelect model={model} readOnly={readOnly}/>
                    </ModelForm>
                </Grid>
                {!model.isNew && <Grid item md={6} xs={12}>
                    <Toolbar><Typography variant={'h4'}>{t`App Info`}</Typography></Toolbar>
                    <Table>
                        <TableBody>
                            <TableRow>
                                <TableCell>{model.attribute('module_name')!.label}</TableCell>
                                <TableCell>{model.module_name}</TableCell>
                            </TableRow>
                            <TableRow>
                                <TableCell>{model.attribute('version')!.label}</TableCell>
                                <TableCell>{model.version}</TableCell>
                            </TableRow>
                            <TableRow>
                                <TableCell>{model.attribute('description')!.label}</TableCell>
                                <TableCell>{model.description}</TableCell>
                            </TableRow>
                            <TableRow>
                                <TableCell>{model.attribute('reachable')!.label}</TableCell>
                                <TableCell>
                                    <Box display={"flex"} alignItems={"center"}>
                                        <AppStatusIcon reachable={!!model.reachable} />
                                        <RestartButton model={model as RxModuleComponentModel} />
                                        <StopButton model={model as RxModuleComponentModel} />
                                    </Box>
                                </TableCell>
                            </TableRow>
                            {model.highestAvailableUpdate && <>
                                <TableRow>
                                    <TableCell>{t`Update Info`}</TableCell>
                                    <TableCell>{(model.updateText as string).split("\n").map((v: string) =>
                                        <React.Fragment
                                            key={v}>{v}<br/></React.Fragment>)}</TableCell>
                                </TableRow>
                                <TableRow>
                                    <TableCell>{t`Perform Update`}</TableCell>
                                    <TableCell><UpdateCell model={model as RxModuleComponentModel}/></TableCell>
                                </TableRow>
                            </>}
                            {model.usesIcon && <TableRow>
                                <TableCell>{t`Icon`}</TableCell>
                                <TableCell>
                                    <Box display={"flex"}>
                                        <img src={model.icon_url as string} alt={model.module_name as string} width={50}/>
                                        <ButtonField text={t`Update Icon`} variant={"outlined"} onClick={updateIcon}/>
                                    </Box>
                                </TableCell>
                            </TableRow>}
                        </TableBody>
                    </Table>
                </Grid>}
            </Grid>
        )

    }
);
