import * as React from 'react';
import {observer, useLocalStore} from "mobx-react-lite";
import {ButtonField, InputField, PaddedPaper} from "@rx-suite/ui";
import {t} from '@lingui/macro';
import {useParams} from 'react-router-dom';
import {UserStore} from "../../stores/UserStore";
import {runInAction} from "mobx";
import {UsersTransport} from "../../transports/UsersTransport";
import {useSnackbar} from "notistack";
import {errToString} from "@rx-suite/data";
import {doLogin, loginState} from "@rx-suite/stores/build";
import {routerStore} from "@rx-suite/data/build";

export const PasswordResetForm: React.FunctionComponent<{}> = observer(() => {
    const {user, token} = useParams();
    const {enqueueSnackbar} = useSnackbar();
    const passwordStore = useLocalStore(current => ({
            password: '',
            passwordConfirmation: '',
            submitPending: false,
            handlePasswordChange(e: React.ChangeEvent<HTMLInputElement>) {
                this.password = e.target.value;
            },
            handleConfirmPasswordChange(e: React.ChangeEvent<HTMLInputElement>) {
                this.passwordConfirmation = e.target.value;
            },
            get passwordsMatch() {
                return this.password === this.passwordConfirmation;
            },
            get formValid() {
                return this.password && this.passwordsMatch;
            },
            async submitForm() {
                runInAction(() => this.submitPending = true);
                try {
                    const response = await (UserStore.transportLayer as UsersTransport).resetPassword(current.user!, current.token!, this.password, this.passwordConfirmation);
                    if (response.error) {
                        current.enqueueSnackbar(t`There was a problem resetting the password: ${errToString(response.error)}`, {variant: 'error'});
                    } else {
                        current.enqueueSnackbar(t`Password reset successfully`, {variant: 'success'});
                        try {
                            const loggedIn = await doLogin(response.name, this.password);
                            if (loggedIn) {
                                routerStore.history.push("/");
                            } else {
                                current.enqueueSnackbar(t`There was a problem resetting the password: ${current.loginState.error}`, {variant: 'error'});
                            }
                        } catch (e) {
                            current.enqueueSnackbar(t`There was a problem logging in: ${errToString(e)}`, {variant: 'error'});
                        }
                    }
                } catch (e) {
                    current.enqueueSnackbar(t`There was a problem resetting the password: ${errToString(e)}`, {variant: 'error'});
                } finally {
                    runInAction(() => this.submitPending = false)
                }

            }
        }), {
            enqueueSnackbar: enqueueSnackbar,
            user: user,
            token: token,
            loginState: loginState
        }
    );
    if (user === undefined || token === undefined) {
        return <div>{t`Invalid password reset link`}</div>
    }
    return (
        <PaddedPaper>
            <InputField
                label={t`Password`}
                value={passwordStore.password}
                type={'password'}
                onChange={passwordStore.handlePasswordChange}
            />
            <InputField
                name={name}
                label={t`Confirm Password`}
                value={passwordStore.passwordConfirmation}
                type={'password'}
                error={!passwordStore.passwordsMatch}
                helperText={passwordStore.passwordsMatch ? undefined : t`Passwords must match`}
                onChange={passwordStore.handleConfirmPasswordChange}
            />
            <ButtonField text={t`Reset Password`}
                         onClick={passwordStore.submitForm}
                         loading={passwordStore.submitPending}
                         disabled={!passwordStore.formValid}/>
        </PaddedPaper>
    )
});
