import {attribute, DataStore, Model, model} from "@rx-suite/data/build";
import {t} from "@lingui/macro";
import * as yup from "yup";

@model({
    transportLayer: "/api/emails",
    permissionTarget: "Email",
    routingInfo: {
        list: '/emails',
        create: '/emails/create',
        read: '/emails/:id/read',
        update: '/emails/:id/edit'
    }
})
export class EmailModel extends Model {

    @attribute({
        label: t`Email Address`,
        description: t`The address to send messages from`,
        validation: yup.string().email(t`Email Address must be a valid email`).required(t`Email Address is a required field`)
    })
    address = '';

    @attribute({
        label: t`Password`,
        description: t`The password to use when authenticating with the email server.`,
        validation: yup.string()
    })
    password: string = '';

    @attribute({
        label: t`Email Server`,
        description: t`The mail server to send messages from.`,
        validation: yup.string().required(t`Server is a required field`)
    })
    server: string = '';

    @attribute({
        label: t`Port`,
        description: t`The port the mail server accepts messages on.`,
        validation: yup.string().required(t`Port is a required field`)
    })
    port?: string = undefined;

    @attribute({
        label: t`System Default Email?`,
        description: t`If checked, any modules without a specific override will use this account.  Checking this will unset the previous system default email.`,
        validation: yup.boolean()
    })
    is_system_default: boolean = false;

    @attribute({
        label: t`Authentication Needed?`,
        description: t`If set, password authentication will be skipped`,
        validation: yup.boolean()
    })
    auth: boolean = true;

    @attribute({
        label: t`Authentication Type`,
        description: t`How to provide credentials to the email server.  Please consult your server's documentation for details.  You can use the "Test Email" button to test your choice.`,
        validation: yup.mixed().when('auth', {
            is: true,
            then: yup.string().required(t`If using Authentication, you must select an Authentication type`),
            otherwise: yup.string().nullable(),
        })
    })
    auth_type: "plain" | "login" | "cram_md5" | null = "login";
}

export const EmailStore = EmailModel.store as unknown as DataStore<EmailModel>;
