import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {RolesTransport} from "../transports/RolesTransport";
import {computed, runInAction} from "mobx";
import {ComponentPermissionModel, ComponentPermissionStore} from "./ComponentPermissionStore";
import {RolePermissionModel, RolePermissionStore} from "./RolePermissionStore";

@model({
    transportLayer: new RolesTransport(),
    permissionTarget: "Role",
    eager: true,
    routingInfo: {
        create: '/roles/create',
        read: '/roles/:id/read',
        update: '/roles/:id/edit'
    }
})

export class RoleModel extends Model {
    @attribute({
        label: t`Name`,
        description: t`The name that will be used to identify this role.`,
        validation: yup.string().required(t`Name is a required field`)
    })
    name: string = '';

    async sendDataWithRules(endPoint: string, json: object) {
        this._state = "updating";
        const rolesTransport = RoleStore.transportLayer as RolesTransport;
        const response = await rolesTransport.sendDataWithRules(endPoint, json);
        runInAction(() => this._state = "done");

        return response;
    }

    // Returns an array of all the component permissions and the associated role permissions
    @computed get permissions() {
        const roleRP = (RolePermissionStore.models || []).filter(rp => rp.role_id === this.id);

        //ensure every component permission has an matching role permission
        return (ComponentPermissionStore.models || []).reduce((acc: Array<Permission>, cp: ComponentPermissionModel) => {
            let rp = roleRP.find((rp: RolePermissionModel) => cp.id === rp.component_permission_id);

            if (!rp) {
                rp = RolePermissionStore.createModel({
                    "component_permission_id": cp.id,
                    "permitted": false
                } as RolePermissionModel)
            }

            acc.push({componentPermission: cp, rolePermission: rp});

            return acc;
        }, []);
    }
}

export type Permission = {
    componentPermission: ComponentPermissionModel
    rolePermission: RolePermissionModel
}

export const RoleStore = RoleModel.store as DataStore<RoleModel>;
