# == Schema Information
#
# Table name: components
#
#  id                :bigint           not null, primary key
#  description       :string
#  has_token         :boolean
#  ip_address        :string
#  is_api            :boolean
#  is_updating       :boolean
#  last_update_error :string
#  module_name       :string
#  port              :integer
#  protocol          :string           default("http")
#  reachable         :boolean
#  token             :string
#  type              :string
#  update_status     :string
#  version           :string
#  created_at        :datetime         not null
#  updated_at        :datetime         not null
#  email_id          :bigint
#  module_id         :string
#
# Indexes
#
#  index_components_on_email_id             (email_id)
#  index_components_on_module_id            (module_id) UNIQUE
#  index_components_on_port_and_ip_address  (port,ip_address) UNIQUE
#
# Foreign Keys
#
#  fk_rails_...  (email_id => emails.id) ON DELETE => nullify
#
class RxModuleComponent < Component
  class << self
    def update_server
      ENV["UPDATE_SERVER"] || RxModule::Licence.licence_server
    end

    # anything the customer is allowed to install (by licence)
    def eligible_installations
      %w[rx_pdf]
    end

    # allowed installations - performed installations
    def available_installations
      eligible_installations.reject do |module_code|
        module_installed?(module_code)
      end
    end

    def module_installed?(module_code)
      exists?(module_id: module_code)
    end

    def module_installable?(module_code)
      available_installations.include?(module_code.to_s)
    end
  end

  def to_partial_path
    Component._to_partial_path
  end

  def upgrade!
    mark_downloading_update

    url = "#{RxModuleComponent.update_server}/package/#{module_id}"
    url += "?prerelease=true" if ENV["ALLOW_PRERELEASE"]
    client = Faraday.new(url: url) do |faraday|
      faraday.use FaradayMiddleware::FollowRedirects
      faraday.adapter Faraday.default_adapter
    end
    response = client.get(url)
    if response.status == 200
      begin
        mark_transferring_update
        Tempfile.create(binmode: true) do |file|
          file.write(response.body)
          conn = Faraday.new(nil, ssl: {
            verify: !RxModule::Communicator.skip_ssl_validation
          }) do |f|
            f.request :multipart
          end
          file_part = Faraday::FilePart.new(file.path, "application/zip")
          update_resp = conn.post(resolve_endpoint("/api/rx_module/update")) do |req|
            req.headers[:Authorization] = "TOKEN #{RxModule::Authorization.current_user_token}"
            req.body = {file: file_part}
          end
          update_json = JSON.parse(update_resp.body)
          Rails.logger.info { {action: "Update response", resp: update_json, component: self} }
          if update_json["error"]
            mark_update_error(update_json["error_details"])
            ping
            raise update_json["error_details"]
          else
            mark_update_applying
          end
          file_part.io.close
          file.close
        end
      rescue Errno::EACCES
        Rails.logger.warn { "Failed to delete tmpfile" }
      end
    else
      mark_update_error("Unreachable update server")
      raise "Unreachable update server"
    end
  end

  def update_icon
    conn = Faraday.new(resolve_endpoint("/api/rx_module/icon"), ssl: {
      verify: !RxModule::Communicator.skip_ssl_validation
    })
    svg_resp = conn.get(nil)
    Tempfile.create(%w[icon_download .svg]) do |f|
      f.write(svg_resp.body)
      f.flush
      f.rewind
      icon_file.attach(io: f, filename: File.basename(f.path), content_type: svg_resp.headers["content-type"], identify: false)
    end
  end

  def update_token
    RxModule::Communicator.call_module_method(module_id, "/api/rx_module/token", :put, {
      token: token
    })
  end

  def ping
    # we restrict the updates to minimize websocket traffic.  We'll only broadcast if something is dirty

    response = RxModule::Communicator.call_module_method(module_id, "/api/rx_module/info", :get)
    process_ping_result(response)
    begin
      if reachable?
        update_icon unless icon_file.attached?

        # this is deliberately checking false because if it's nil,
        # the module doesn't support tokens yet
        update_token if has_token == false
      end
    rescue => err
      Rails.logger.warn { {msg: "Issue updating module icon", component: as_json} }
    end
  rescue => err
    Rails.logger.info { {msg: "Component may be offline or returning invalid responses", component: as_json} }
    update(reachable: false)
  end

  def process_ping_result(status)
    updates = {}
    updates[:reachable] = true unless self[:reachable]
    {
      module_id: :id,
      module_name: :name,
      description: :description,
      version: :version,
      is_api: :is_api
    }.each do |k_this, k_resp|
      updates[k_this] = status[k_resp] unless self[k_this] == status[k_resp]
    end

    # If this is present, this means the module itself has support for tokens
    # If it's nil, we want to keep has_token nil, not false, so we can tell the difference
    unless status[:has_token].nil?
      updates[:has_token] = status[:has_token]
    end

    if status[:last_update]
      updates[:is_updating] = status[:last_update][:in_progress] unless self[:is_updating] == status[:last_update][:in_progress]
      updates[:last_update_error] = status[:last_update][:error_msg] unless self[:last_update_error] == status[:last_update][:error_msg]
    else
      updates[:is_updating] = false unless self[:is_updating] == false
      updates[:last_update_error] = nil unless self[:last_update_error].nil?
    end
    if status[:ssl]
      updates[:protocol] = "https" unless self[:protocol] == "https"
    else
      updates[:protocol] = "http" unless self[:protocol] == "http"
    end

    update(updates) if updates.size > 0

    permissions_to_keep = []
    status[:permissions]&.each do |target_definition|
      target_definition[:actions].each do |action_definition|
        permission = component_permissions.where(target_id: target_definition[:id], action_id: action_definition[:id]).first
        permission_allowed = true
        if action_definition[:feature_subfeature_id] && !RxModule::Licence.feature_allowed?(action_definition[:feature_module_id], action_definition[:feature_subfeature_id])
          permission_allowed = false
          if permission
            Rails.logger.info { "Feature not allowed (#{permission[:feature_module_id]}, #{permission[:feature_subfeature_id]}) - revoking permission #{target_definition[:name]}:#{action_definition[:name]} from all roles" }
            Role.revoke_from_all(permission)
          end
        elsif action_definition[:feature_module_id] && !RxModule::Licence.module_allowed?(action_definition[:feature_module_id])
          permission_allowed = false
          if permission
            Rails.logger.info { "Feature not allowed (#{permission[:feature_module_id]}, #{permission[:feature_subfeature_id]}) - revoking permission #{target_definition[:name]}:#{action_definition[:name]} from all roles" }
            Role.revoke_from_all(permission)
          end
        end
        if permission_allowed
          permission ||= component_permissions.create(target_id: target_definition[:id], action_id: action_definition[:id])
          updates = {}
          if permission[:target_display_name] != target_definition[:name]
            updates[:target_display_name] = target_definition[:name]
          end
          if permission[:action_display_name] != action_definition[:name]
            updates[:action_display_name] = action_definition[:name]
          end
          if permission[:target_category] != target_definition[:category]
            updates[:target_category] = target_definition[:category]
          end
          if permission[:feature_module_id] != action_definition[:feature_module_id]
            updates[:feature_module_id] = action_definition[:feature_module_id]
          end
          if permission[:feature_subfeature_id] != action_definition[:feature_subfeature_id]
            updates[:feature_subfeature_id] = action_definition[:feature_subfeature_id]
          end
          permission.update(updates) if updates.size > 0
          permissions_to_keep.push(permission.id)
        end
      end
    end
    component_permissions.where.not(id: permissions_to_keep).destroy_all
  end
end
